
#include	"config_stc.h"
#include	"ASCII5x7.h"
#include	"ASCII-10x24.h"


/*************	功能说明	**************

淘宝买的OLED屏驱动程序. 驱动IC为SSD1306, I2C接口.

支持中文英文混显, 支持10x24点大数字显示.

******************************************/




//************************************************************************************************


/*	定义接口	*/
					// 1--GND				D2	SDAout		D/C --> SA0 从机地址最低位, 模块接到0了
					// 2--VCC	3.3V		E ----> low		RES --> 复位脚, 低电平复位
sbit SCL = P2^0;	// 3--D0	SCL			R/W --> low
sbit SDA = P2^1;	// 4--D1	SDAin		CS ---> low

#define SLAW	0x78	/* 从机地址写 */
#define SLAR	0x79	/* 从机地址读 */

//========================================================================
// 描述: 延时函数
// 参数: dly 延时时间ms, 1~65535.
// 返回: 无
//========================================================================
void	OLED_delay_ms(u16 ms)	// 1~65535
{
	u16 i;
	do
	{
		i = MAIN_Fosc / 10000;		//STC8系列
		while(--i)	;
	}while(--ms);
}


/****************************/

void	OLED_I2C_Delay(void)
{
	u8	dly;
	dly = (MAIN_Fosc * 2) / 3000000UL;	//  周期数: 6+3*n,   I2C位时间调整, (Fosc/1000000) * n / 3 = (MAIN_Fosc * n) / 3000000UL, n为us, 3为while(--dly)的周期数
	while(--dly)	;
}

#define	OLED_I2C_NOP()		NOP(3)		// 至少加3个NOP @24MHz, 至少加4个NOP @36MHz, 至少加5个NOP @44MHz, 否则显示不正常 STC8H8K64U


/****************************/
void OLED_I2C_Start(void)               //start the I2C, SDA High-to-low when SCL is high
{
	SDA = 1;
//	OLED_I2C_Delay();
	SCL = 1;
	OLED_I2C_Delay();
	SDA = 0;
	OLED_I2C_Delay();
	SCL = 0;
//	OLED_I2C_Delay();
}


void OLED_I2C_Stop(void)		//STOP the I2C, SDA Low-to-high when SCL is high
{
	SDA = 0;
	OLED_I2C_Delay();
	SCL = 1;
	OLED_I2C_Delay();
	SDA = 1;
	OLED_I2C_Delay();
}


/********** 写一个字节并读出应答放F0(正确应答F0=0, 错误F0=1) ******************/
void OLED_I2C_WriteAbyte(u8 dat)		//write a byte to I2C	速度: 24MHZ, 12T/bit, 107+8=115T, 4.8us/byte.
{
	B = dat;

	SDA = B7;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B6;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B5;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B4;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B3;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B2;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B1;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
	SDA = B0;	OLED_I2C_NOP();	SCL = 1;	OLED_I2C_NOP();	SCL = 0;	OLED_I2C_NOP();
/*
	SDA = B7;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B6;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B5;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B4;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B3;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B2;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B1;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
	SDA = B0;	OLED_I2C_NOP();	SCL = 1;	NOP(2);	SCL = 0;	OLED_I2C_NOP();
*/

	SDA = 1;
	OLED_I2C_NOP();
	SCL = 1;
	OLED_I2C_NOP();
	F0 = SDA;
	SCL = 0;
//	OLED_I2C_NOP();
}


void	OLED_WriteCMD(u8 cmd)	// 写命令cmd
{
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x00);	//Write command, C0=0, D/C=0, follow by six '0'
		if(!F0)
		{
			OLED_I2C_WriteAbyte(cmd);	//command
		}
	}
	OLED_I2C_Stop();
}


//========================================================================
// 函数: void Set_Dot_Addr_LCD(int x,int y)
// 描述: 设置在LCD的真实坐标系上的X、Y点对应的RAM地址
// 参数: x 		X轴坐标
//		 y 		Y轴坐标
// 返回: 无
// 备注: 仅设置当前操作地址，为后面的连续操作作好准备
// 版本:
//      2007/04/10      First version
//========================================================================
void Set_Dot_Addr(u8 x, u8 y)	//x为横向的点0~127, y为纵向的页0~7
{
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);				//Slave address,SA0=0
	OLED_I2C_WriteAbyte(0x00);				//Write command, C0=0, D/C=0, follow by six '0'
	OLED_I2C_WriteAbyte(0xb0 + y);			//设置页0~7
	OLED_I2C_WriteAbyte(0x00);				//Write command, C0=0, D/C=0, follow by six '0'
	OLED_I2C_WriteAbyte((x >> 4)  | 0x10);	//设置列0~127 高nibble
	OLED_I2C_WriteAbyte(0x00);				//Write command, C0=0, D/C=0, follow by six '0'
	OLED_I2C_WriteAbyte(x & 0x0f);			//设置列0~127 低nibble
	OLED_I2C_Stop();

}


//******************************************
void FillPage(u8 y, u8 color)		//Fill Page LCD RAM, y为页码 0~7
{
	u8 i;
	Set_Dot_Addr(0,y);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<128; i++)	OLED_I2C_WriteAbyte(color);
		}
	}
	OLED_I2C_Stop();
}

//******************************************
void FillAll(u8 color)		//Fill all LCD RAM， color为填充的颜色
{
	u8 i;
	for(i=0; i<8; i++)	FillPage(i,color);
}

//******************************************
void OLED_config(void)		//初始化函数
{
//	P1.0--OLED12864-SCL 开漏
//	P1.1--OLED12864-SDA 开漏
	P1n_open_drain(Pin1+Pin0);

	SCL = 1;
	SDA = 1;
	OLED_delay_ms(10);

	OLED_WriteCMD(0xAE);     //Set Display Off

	OLED_WriteCMD(0xd5);     //display divide ratio/osc. freq. mode
	OLED_WriteCMD(0x80);     //

	OLED_WriteCMD(0xA8);     //multiplex ration mode:63
	OLED_WriteCMD(0x3F);

	OLED_WriteCMD(0xD3);     //Set Display Offset
	OLED_WriteCMD(0x00);

	OLED_WriteCMD(0x40);     //Set Display Start Line

	OLED_WriteCMD(0x8D);     //Set Display Offset
	OLED_WriteCMD(0x14);

	OLED_WriteCMD(0xA1);     //Segment Remap

	OLED_WriteCMD(0xC8);     //Sst COM Output Scan Direction

	OLED_WriteCMD(0xDA);     //common pads hardware: alternative
	OLED_WriteCMD(0x12);

	OLED_WriteCMD(0x81);     //contrast control
	OLED_WriteCMD(0xCF);

	OLED_WriteCMD(0xD9);	    //set pre-charge period
	OLED_WriteCMD(0xF1);

	OLED_WriteCMD(0xDB);     //VCOM deselect level mode
	OLED_WriteCMD(0x40);	    //set Vvcomh=0.83*Vcc

	OLED_WriteCMD(0xA4);     //Set Entire Display On/Off

	OLED_WriteCMD(0xA6);     //Set Normal Display

	FillAll(0);
	OLED_WriteCMD(0xAF);     //Set Display On

}


//******************************************
/*
void WriteAscii_6x8(u8 x, u8 y, u8 chr)	//向指定位置写一个ASCII码字符, x为横向的点0~127, y为纵向的页0~7, chr为要写的字符
{
	u8 code *p;
	u8 i;

	if(x > (128-5))	return;
	if(y >= 8)		return;
	p = (u16)chr * 6 + ASCII5x7;

	Set_Dot_Addr(x,y);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<6; i++)
			{
				OLED_I2C_WriteAbyte(*p);
				p++;
			}
		}
	}
	OLED_I2C_Stop();
}
*/


/************ 打印ASCII字符串 *************************/
void	printf_ascii_6x8(u8 x, u8 y, u8 *ptr)	//x为横向的点0~127, y为纵向的页0~7, *ptr为要打印的字符串指针.  打印21个ASCII码(21*6+9=135个字节)耗时625us@44MHZ, 652us@36MHz, 850us@24MHz
{
    u8 c;
	u8	i;
	u8 code *p;

	if(y >= 8)	return;
	Set_Dot_Addr(x,y);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for (;;)
			{
				c = *ptr;
				if(c == 0)		break;	//遇到停止符0结束
				if(c >= 0xa0)	break;	//非ASCII码结束

				p = (u16)c * 6 + ASCII5x7;
				for(i=0; i<6; i++)
				{
					OLED_I2C_WriteAbyte(*p);
					p++;
				}
				x += 6;
				ptr++;
				if(x > (128-5))	break;
			}
		}
	}
	OLED_I2C_Stop();
}

//******************************************
/*
void WriteAscii_8x16(u8 x, u8 y, u8 chr)	//向指定位置写一个ASCII码字符, x为横向的点0~127, y为纵向的页0~7, chr为要写的字符
{
	u8 code *p;
	u8 i;

	if(x > (128-8))		return;
	if(y > 6)			return;
		 if(chr == 'A')		chr = 0;
	else if(chr == 'V')		chr = 1;
	else if(chr == 'W')		chr = 2;
	else	return;

	p = (u16)chr * 16 + ASCII8x16;

	Set_Dot_Addr(x,y);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<8; i++)
			{
				OLED_I2C_WriteAbyte(*p);
				p++;
			}
		}
	}
	OLED_I2C_Stop();

	Set_Dot_Addr(x,y+1);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<8; i++)
			{
				OLED_I2C_WriteAbyte(*p);
				p++;
			}
		}
	}
	OLED_I2C_Stop();
}
*/


//******************************************
void WriteAscii_10x24(u8 x, u8 y, u8 chr)	//向指定位置写一个ASCII码字符, x为横向的点0~127, y为纵向的页0~7, chr为要写的字符
{
	u8 code *p;
	u8 i;

	if(x > (128-10))	return;
	if(y >= 6)			return;
	p = (u16)chr * 30 + ASCII10x24;

	Set_Dot_Addr(x,y);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<10; i++)
			{
				OLED_I2C_WriteAbyte(*p);
				p++;
			}
		}
	}
	OLED_I2C_Stop();

	Set_Dot_Addr(x,y+1);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<10; i++)
			{
				OLED_I2C_WriteAbyte(*p);
				p++;
			}
		}
	}
	OLED_I2C_Stop();

	Set_Dot_Addr(x,y+2);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			for(i=0; i<10; i++)
			{
				OLED_I2C_WriteAbyte(*p);
				p++;
			}
		}
	}
	OLED_I2C_Stop();
}

//******************************************
void WriteDot_3x3(u8 x, u8 y)	//向指定位置写一个小数点,  x为横向的点0~127, y为纵向的页0~7
{
	if(x > (128-3))	return;
	if(y >= 8)		return;

	Set_Dot_Addr(x,y);
	OLED_I2C_Start();
	OLED_I2C_WriteAbyte(SLAW);		//Slave address,SA0=0
	if(!F0)
	{
		OLED_I2C_WriteAbyte(0x40);	//write data,  C0=0, D/C=1, follow by six '0'
		if(!F0)
		{
			OLED_I2C_WriteAbyte(0x38);
			OLED_I2C_WriteAbyte(0x38);
			OLED_I2C_WriteAbyte(0x38);
		}
	}
	OLED_I2C_Stop();
}

//************ 打印ASCII 10x24英文字符串 *************************
void	printf_ascii_10x24(u8 x, u8 y, u8 *ptr)	//x为横向的点0~127, y为纵向的页0~7, *ptr为要打印的字符串指针, 间隔2点.
{
    u8 c;

	for (;;)
	{
        c = *ptr;
		if(c == 0)		return;	//遇到停止符0结束
		if((c >= '0') && (c <= '9'))			//ASCII码
		{
			WriteAscii_10x24(x,y,c-'0');
			x += 12;
			ptr++;
		}
		else if(c == '.')
		{
			WriteDot_3x3(x,y+2);
			x += 6;
			ptr++;
		}
		else if(c == ' ')	//显示空格
		{
			WriteAscii_10x24(x,y,11);
			x += 12;
			ptr++;
		}
		else if(c == '-')	//显示空格
		{
			WriteAscii_10x24(x,y,10);
			x += 12;
			ptr++;
		}
	}
}

