#include <stdio.h>
#include <string.h>
#include "config_stc.h"
#include "key.h"
#include "timer_alarm.h"
#include "pwm_out.h"
#include "OLED128x64.h"

#define PWM_DUTY_STEP (uint16_t)(25)

volatile uint16_t gIs_Key0_Pressed;   
volatile uint16_t gKey0_Pressed_Cnt;  
volatile uint16_t gPWM_Duty;  // [0~100]%

void Global_Init(void);

void main(void)
{
	char sbuf[22] = {0};
	gIs_Key0_Pressed = 0;
	gKey0_Pressed_Cnt = 0;
	gPWM_Duty = PWM_DUTY_STEP;

	/* 0. 初始化 */
	Global_Init();
	Key0_Init();
	Timer2_Alarm_Init();
	PWMA_Out_Init();
	OLED_config();
	printf_ascii_10x24(40,5,"  ");
	FillAll(0);

	/* 1. 启动 */
	Timer2_Alarm_Start();
	PWMA_Out_Start(gPWM_Duty);

	/* 2. 打印标题 */
	memset(sbuf, sizeof(sbuf), 0);
	sprintf(sbuf, "[v1]PWM LED Test");
	printf_ascii_6x8(0, 0, sbuf);
	
	memset(sbuf, sizeof(sbuf), 0);
	sprintf(sbuf, "PWM Duty[%%]%03u", gPWM_Duty);
	printf_ascii_6x8(0, 2, sbuf);
	
	memset(sbuf, sizeof(sbuf), 0);
	sprintf(sbuf, "KeyPressCnt[%u]", gKey0_Pressed_Cnt);
	printf_ascii_6x8(0, 5, sbuf);

	while(1)
	{  
        if(gIs_Key0_Pressed)
        {
			++gKey0_Pressed_Cnt;
			
            gPWM_Duty += 25;
            if(gPWM_Duty > 100)
			{
                gPWM_Duty = 0;
			}
            
            PWMA_Out_UpdateDuty(gPWM_Duty);
            
            memset(sbuf, sizeof(sbuf), 0);
            sprintf(sbuf, "PWM Duty[%%]%03u", gPWM_Duty);
            printf_ascii_6x8(0, 2, sbuf);
			
			memset(sbuf, sizeof(sbuf), 0);
            sprintf(sbuf, "KeyPressCnt[%u]", gKey0_Pressed_Cnt);
            printf_ascii_6x8(0, 5, sbuf);
	
			gIs_Key0_Pressed = 0;
        }
	}
}

void Global_Init(void)
{
    P_SW2 |= 0x80;  // 使能访问XFR，没有冲突不用关闭

    P0M0 = 0x00;
    P0M1 = 0x00;
    P1M0 = 0x00;
    P1M1 = 0x00;
    P2M0 = 0x00;
    P2M1 = 0x00;
    P3M0 = 0x00;
    P3M1 = 0x00;
    P4M0 = 0x00;
    P4M1 = 0x00;
    P5M0 = 0x00;
    P5M1 = 0x00;
}

void Timer2_Routine() interrupt 12
{
    if(Is_Key0_Pressed())
    {
        gIs_Key0_Pressed = 1;
    }
}
